@@ -1,6 +1,7 @@
 #pragma once
 
 #include <panda/refcnt.h>
+#include <iostream>
 
 namespace panda {
 
@@ -13,7 +14,7 @@ template <typename Ret, typename... Args>
 struct Ifunction : virtual public RefCounted {
     virtual ~Ifunction() {}
     virtual Ret operator()(Args...) = 0;
-    virtual bool equals(Ifunction* oth) const = 0;
+    virtual bool equals(const Ifunction* oth) const = 0;
 };
 
 template<typename T, bool Trivial = std::is_class<T>::value>
@@ -94,9 +95,11 @@ public:
         return func(args...);
     }
 
-    bool equals(Ifunction<Ret, Args...>* oth) const override {
-        auto foth = dynamic_cast<abstract_function*>(oth);
-        if (foth == nullptr) return false;
+    bool equals(const Ifunction<Ret, Args...>* oth) const override {
+        auto foth = dynamic_cast<const abstract_function*>(oth);
+        if (foth == nullptr) {
+            return false;
+        }
 
         return func == foth->func;
     }
@@ -106,21 +109,20 @@ public:
 
 template <typename Func, typename Ret, typename... Args>
 class abstract_function<Func, Ret, false, Args...> : public Ifunction<Ret, Args...>
-        , public callable<Func, Ret, !has_call_operator<Func, Args...>::value, function<Ret(Args...)>, Args...>
+        , public callable<Func, Ret, !has_call_operator<Func, Args...>::value, Ifunction<Ret, Args...>, Args...>
 {
 public:
     using Derfed = typename std::remove_reference<Func>::type;
-    using Caller = callable<Func, Ret, !has_call_operator<Func, Args...>::value, function<Ret(Args...)>, Args...>;
+    using Caller = callable<Func, Ret, !has_call_operator<Func, Args...>::value, Ifunction<Ret, Args...>, Args...>;
 
     template <typename F>
     explicit abstract_function(F&& f) : Caller(std::forward<F>(f)) {}
 
     Ret operator()(Args... args) override {
-        function<Ret(Args...)> self(this);
-        return Caller::operator ()(self, args...);
+        return Caller::operator()(*this, args...);
     }
 
-    bool equals(Ifunction<Ret, Args...>* oth) const override {
+    bool equals(const Ifunction<Ret, Args...>* oth) const override {
         return static_cast<const Ifunction<Ret, Args...>*>(this) == oth;
     }
 
@@ -147,8 +149,8 @@ public:
         return func(args...);
     }
 
-    bool equals(Ifunction<Ret, Args...>* oth) const override {
-        auto foth = dynamic_cast<abstract_function*>(oth);
+    bool equals(const Ifunction<Ret, Args...>* oth) const override {
+        auto foth = dynamic_cast<const abstract_function*>(oth);
         if (foth == nullptr) return false;
 
         return func == foth->func;
@@ -162,19 +164,42 @@ auto make_abstract_function(Ret (*f)(Args...)) -> shared_ptr<abstract_function<R
     return panda::make_shared<abstract_function<Ret (*)(Args...), Ret, true, Args...>>(f);
 }
 
-template <typename Ret, typename... Args,
-          typename Functor, bool IsComp = is_comparable<typename std::remove_reference<Functor>::type>::value,
-          typename Check = decltype(std::declval<Functor>()(std::declval<Args>()...))>
-shared_ptr<abstract_function<Functor, Ret, IsComp, Args...>> make_abstract_function(Functor&& f, Check(*)() = 0) {
-    return panda::make_shared<abstract_function<Functor, Ret, IsComp, Args...>>(std::forward<Functor>(f));
+template <typename Ret, typename... Args>
+auto tmp_abstract_function(Ret (*f)(Args...)) -> abstract_function<Ret (*)(Args...), Ret, true, Args...> {
+    return abstract_function<Ret (*)(Args...), Ret, true, Args...>(f);
 }
 
- template <typename Ret, typename... Args,
-           typename Functor, bool IsComp = is_comparable<typename std::remove_reference<Functor>::type>::value,
-           typename Check = decltype(std::declval<Functor>()(function<Ret(Args...)>(), std::declval<Args>()...))>
- shared_ptr<abstract_function<Functor, Ret, IsComp, Args...>> make_abstract_function(Functor&& f) {
-     return panda::make_shared<abstract_function<Functor, Ret, IsComp, Args...>>(std::forward<Functor>(f));
- }
+template <typename Ret, typename... Args,
+          typename Functor, bool IsComp = is_comparable<typename std::remove_reference<Functor>::type>::value,
+          typename DeFunctor = typename std::remove_reference<Functor>::type,
+          typename Check = decltype(std::declval<Functor>()(std::declval<Args>()...))>
+shared_ptr<abstract_function<DeFunctor, Ret, IsComp, Args...>> make_abstract_function(Functor&& f, Check(*)() = 0) {
+    return panda::make_shared<abstract_function<DeFunctor, Ret, IsComp, Args...>>(std::forward<Functor>(f));
+}
+
+template <typename Ret, typename... Args,
+          typename Functor, bool IsComp = is_comparable<typename std::remove_reference<Functor>::type>::value,
+          typename DeFunctor = typename std::remove_reference<Functor>::type,
+          typename Check = decltype(std::declval<Functor>()(std::declval<Args>()...))>
+abstract_function<DeFunctor, Ret, IsComp, Args...> tmp_abstract_function(Functor&& f, Check(*)() = 0) {
+    return abstract_function<DeFunctor, Ret, IsComp, Args...>(std::forward<Functor>(f));
+}
+
+template <typename Ret, typename... Args,
+          typename Functor, bool IsComp = is_comparable<typename std::remove_reference<Functor>::type>::value,
+          typename DeFunctor = typename std::remove_reference<Functor>::type,
+          typename Check = decltype(std::declval<Functor>()(std::declval<Ifunction<Ret, Args...>&>(), std::declval<Args>()...))>
+shared_ptr<abstract_function<DeFunctor, Ret, IsComp, Args...>> make_abstract_function(Functor&& f) {
+    return panda::make_shared<abstract_function<DeFunctor, Ret, IsComp, Args...>>(std::forward<Functor>(f));
+}
+
+template <typename Ret, typename... Args,
+          typename Functor, bool IsComp = is_comparable<typename std::remove_reference<Functor>::type>::value,
+          typename DeFunctor = typename std::remove_reference<Functor>::type,
+          typename Check = decltype(std::declval<Functor>()(std::declval<Ifunction<Ret, Args...>&>(), std::declval<Args>()...))>
+abstract_function<DeFunctor, Ret, IsComp, Args...> tmp_abstract_function(Functor&& f) {
+    return abstract_function<DeFunctor, Ret, IsComp, Args...>(std::forward<Functor>(f));
+}
 
 template <class Class, typename Ret, typename... Args>
 struct method : public Ifunction<Ret, Args...>{
@@ -191,8 +216,8 @@ struct method : public Ifunction<Ret, Args...>{
         return (thiz.get()->*meth)(std::forward<Args>(args)...);
     }
 
-    bool equals(ifunction* oth) const override {
-        auto moth = dynamic_cast<method<Class, Ret, Args...>*>(oth);
+    bool equals(const ifunction* oth) const override {
+        auto moth = dynamic_cast<const method<Class, Ret, Args...>*>(oth);
         if (moth == nullptr) return false;
 
         return operator ==(*moth);
@@ -224,4 +249,9 @@ template <typename Ret, typename... Args, class Class>
 inline shared_ptr<method<Class, Ret, Args...>> make_abstract_function(Ret (Class::*meth)(Args...), shared_ptr<Class> thiz = nullptr) {
     return panda::make_shared<method<Class, Ret, Args...>>(meth, thiz);
 }
+
+template <typename Ret, typename... Args, class Class>
+inline method<Class, Ret, Args...> tmp_abstract_function(Ret (Class::*meth)(Args...), shared_ptr<Class> thiz = nullptr) {
+    return method<Class, Ret, Args...>(meth, thiz);
+}
 }
